@@ -103,11 +103,16 @@ class Dataset(BaseNode):
       return 0
 
     if not os.path.exists(os.path.join(self.dir, 'unlabeled_list.txt')):
+      unorder_list = []
+      for file in os.listdir(unlabeled_folder):
+        if file[0] == '.':
+          continue
+        unorder_list.append(file)
+      
+      order_list = sorted(unorder_list)
       with open(os.path.join(self.dir, 'unlabeled_list.txt'), 'w') as fp:
-        for file in os.listdir(unlabeled_folder):
-          if file[0] == '.':
-            continue
-          fp.write('%s,%d\n' % (file, 0))
+        for file in order_list:
+          fp.write('%s/%s,%d\n' % (self.unlabeled_tag, file, 0))
 
     has_labeled_list = []
     if os.path.exists(os.path.join(self.dir, 'candidates.txt')):
@@ -127,7 +132,7 @@ class Dataset(BaseNode):
       line_content = fp.readline()
       while line_content:
         file_name, _ = line_content.split(',')
-        if file_name not in has_labeled_list:
+        if file_name.split('/')[-1] not in has_labeled_list:
           unlabeled_list.append(file_name)
 
         line_content = fp.readline()
@@ -140,11 +145,16 @@ class Dataset(BaseNode):
       return None
 
     if not os.path.exists(os.path.join(self.dir, 'unlabeled_list.txt')):
+      unorder_list = []
+      for file in os.listdir(unlabeled_folder):
+        if file[0] == '.':
+          continue
+        unorder_list.append(file)
+      
+      order_list = sorted(unorder_list)
       with open(os.path.join(self.dir, 'unlabeled_list.txt'), 'w') as fp:
-        for file in os.listdir(unlabeled_folder):
-          if file[0] == '.':
-            continue
-          fp.write('%s,%d\n' % (file, 0))
+        for file in order_list:
+          fp.write('%s/%s,%d\n' % (self.unlabeled_tag, file, 0))
 
     has_labeled_list = []
     if os.path.exists(os.path.join(self.dir, 'candidates.txt')):
@@ -170,8 +180,8 @@ class Dataset(BaseNode):
     
     unlabeled_data = []
     for index, file in enumerate(unlabeled_list):
-      if file not in has_labeled_list:
-        unlabeled_data.append((os.path.join(self.dir, self.unlabeled_tag, file), {'id': index, 'file_id': file}))
+      if file.split('/')[-1] not in has_labeled_list:
+        unlabeled_data.append({'id': index, 'file_id': file})
 
     return unlabeled_data
 
@@ -185,13 +195,13 @@ class Dataset(BaseNode):
     with open(os.path.join(self.dir, 'candidates.txt'), 'a') as fp:
       if status == 'OK':
         # copy data file to candidates/data
-        shutil.copy(os.path.join(self.dir, self.unlabeled_tag, unlabeled_file), os.path.join(self.dir, 'candidates', 'data'))
+        shutil.copy(os.path.join(self.dir, unlabeled_file), os.path.join(self.dir, 'candidates', 'data'))
 
         # copy label file to candidates/label
         shutil.copy(label_file, os.path.join(self.dir, 'candidates', 'label'))
 
         # write to canndidates.txt
-        fp.write('%d,%s,%s\n'%(unlabeled_id, 'candidates/data/%s'%unlabeled_file, 'candidates/label/%s'%label_file.split('/')[-1]))
+        fp.write('%d,%s,%s\n'%(unlabeled_id, 'candidates/data/%s'%unlabeled_file.split('/')[-1], 'candidates/label/%s'%label_file.split('/')[-1]))
 
   def check_candidate(self, unlabeled_files, finished_label_folder):
     # 检查准备进入候选列表的标注数据，与等待的未标注数据一致
@@ -812,19 +822,70 @@ class Dataset(BaseNode):
 class UnlabeledDataset(Dataset):
   def __init__(self, dataset):
     super(UnlabeledDataset, self).__init__()
-    self.data = dataset.unlabeled()
+    self.dataset = dataset
+    self.unlabeled_data = dataset.unlabeled()
 
   def data_pool(self):
     for id in range(self.size):
       yield self.at(id)
 
-  def at(self, id):
-    print(id)
-    return self.data[id]
+  def at(self, unlabeled_id):
+    id = self.unlabeled_data[unlabeled_id]['id']
+    file_id = self.unlabeled_data[unlabeled_id]['file_id']
+    data = self.dataset.at(id, file_id)
+    data[1].update({'file_id': file_id, 'id': id})
+    return data
 
   @property
   def size(self):
-    return len(self.data)
+    return len(self.unlabeled_data)
+
+
+class CandidateDataset(Dataset):
+  def __init__(self, dataset):
+    super(CandidateDataset, self).__init__()
+    self.dataset = dataset
+    self.candidates = []
+
+    # 使用candidate文件夹中的数据(已经完成标注的数据)
+    if os.path.exists(os.path.join(self.dataset.dir, 'candidates.txt')):
+      with open(os.path.join(self.dataset.dir, 'candidates.txt'), 'r') as fp:
+        content = fp.readline()
+        content = content.strip()
+
+        while content != '':
+          id, file_id, annotation_file_id = content.split(',')
+
+          self.candidates.append({
+            'id': id,
+            'file_id': file_id,
+            'annotation_file_id': annotation_file_id
+          })
+          content = fp.readline()
+          content = content.strip()
+
+  def data_pool(self):
+    for id in range(self.size):
+      yield self.at(id)
+
+  def at(self, local_id):
+    id = self.candidates[local_id]['id']
+    file_id = self.candidates[local_id]['file_id']
+    annotation_file_id = self.candidates[local_id]['annotation_file_id']
+ 
+    data = self.dataset.at((int)(id), file_id)
+    data[1].update({'file_id': file_id, 'id': id, 'annotation_file_id': os.path.join(self.dataset.dir, annotation_file_id)})
+
+    return data
+
+  def split(self, split_params, split_method):
+    _, b = self.dataset.split(split_params, split_method)
+    return self, b
+
+  @property
+  def size(self):
+    return len(self.candidates)
+
 
 class DataAnnotationSplitDataset(object):
   def __init__(self, dataset):
