@@ -7,8 +7,8 @@ from __future__ import print_function
 from __future__ import unicode_literals
 import sys
 from typing import NamedTuple
-sys.path.append('/workspace/workspace/portrait_code/tool/antgo')
-sys.path.append('/workspace/workspace/portrait_code/tool/antgo/antgo')
+# sys.path.append('/workspace/workspace/portrait_code/tool/antgo')
+# sys.path.append('/workspace/workspace/portrait_code/tool/antgo/antgo')
 from antgo.ant.generate import *
 from antgo.ant.demo import *
 from antgo.ant.train import *
